@@ -288,6 +288,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
   GRN_TEXT_INIT(&ctx->impl->subbuf, 0);
   ctx->impl->edge = NULL;
   grn_loader_init(&ctx->impl->loader);
+  ctx->impl->module_path = NULL;
 }
 
 void
@@ -943,7 +944,7 @@ grn_ctx_sendv(grn_ctx *ctx, int argc, char **argv, int flags)
 }
 
 unsigned int
-grn_ctx_send(grn_ctx *ctx, char *str, unsigned int str_len, int flags)
+grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
 {
   if (!ctx) { return 0; }
   GRN_API_ENTER;
@@ -1275,7 +1276,7 @@ exit :
 }
 
 void
-grn_cache_expire(uint32_t size)
+grn_cache_expire(int32_t size)
 {
   grn_cache_entry *ce0 = (grn_cache_entry *)&grn_gcache;
   MUTEX_LOCK(grn_gcache.mutex);
@@ -1984,7 +1985,7 @@ grn_get_package(void)
   return PACKAGE;
 }
 
-#ifdef HAVE_SIGNAL_H
+#if defined(HAVE_SIGNAL_H) && !defined(WIN32)
 static int segv_received = 0;
 static void
 segv_handler(int signal_number, siginfo_t *info, void *context)
@@ -2019,13 +2020,13 @@ segv_handler(int signal_number, siginfo_t *info, void *context)
   GRN_LOG(ctx, GRN_LOG_CRIT, "----------------");
   abort();
 }
-#endif /* HAVE_SIGNAL_H */
+#endif /* defined(HAVE_SIGNAL_H) && !defined(WIN32) */
 
 grn_rc
 grn_set_segv_handler(void)
 {
   grn_rc rc = GRN_SUCCESS;
-#ifdef HAVE_SIGNAL_H
+#if defined(HAVE_SIGNAL_H) && !defined(WIN32)
   grn_ctx *ctx = &grn_gctx;
   struct sigaction action;
 
@@ -2040,3 +2041,32 @@ grn_set_segv_handler(void)
 #endif
   return rc;
 }
+
+#if defined(HAVE_SIGNAL_H) && !defined(WIN32)
+static struct sigaction old_int_handler;
+static void
+int_handler(int signal_number, siginfo_t *info, void *context)
+{
+  grn_gctx.stat = GRN_CTX_QUIT;
+  sigaction(signal_number, &old_int_handler, NULL);
+}
+#endif /* defined(HAVE_SIGNAL_H) && !defined(WIN32) */
+
+grn_rc
+grn_set_int_handler(void)
+{
+  grn_rc rc = GRN_SUCCESS;
+#if defined(HAVE_SIGNAL_H) && !defined(WIN32)
+  grn_ctx *ctx = &grn_gctx;
+  struct sigaction action;
+
+  sigemptyset(&action.sa_mask);
+  action.sa_sigaction = int_handler;
+
+  if (sigaction(SIGINT, &action, &old_int_handler)) {
+    SERR("failed to set SIGINT action");
+    rc = ctx->rc;
+  };
+#endif
+  return rc;
+}
