@@ -92,7 +92,7 @@ extern "C" {
 #define BACKTRACE(ctx)
 #endif /* HAVE_EXECINFO_H */
 
-void grn_ctx_impl_err(grn_ctx *ctx);
+GRN_API void grn_ctx_impl_err(grn_ctx *ctx);
 
 #ifdef HAVE_EXECINFO_H
 #define LOGTRACE(ctx,lvl) {\
@@ -346,10 +346,10 @@ char *grn_strdup(grn_ctx *ctx, const char *s, const char* file, int line, const
 #  define grn_strdup grn_strdup_default
 #endif
 
-void *grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const char *func);
+GRN_API void *grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const char *func);
 void *grn_calloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const char *func);
 void *grn_realloc_default(grn_ctx *ctx, void *ptr, size_t size, const char* file, int line, const char *func);
-char *grn_strdup_default(grn_ctx *ctx, const char *s, const char* file, int line, const char *func);
+GRN_API char *grn_strdup_default(grn_ctx *ctx, const char *s, const char* file, int line, const char *func);
 
 #ifdef USE_FAIL_MALLOC
 int grn_fail_malloc_check(size_t size, const char *file, int line, const char *func);
@@ -359,19 +359,19 @@ void *grn_realloc_fail(grn_ctx *ctx, void *ptr, size_t size, const char* file, i
 char *grn_strdup_fail(grn_ctx *ctx, const char *s, const char* file, int line, const char *func);
 #endif
 
-void grn_free(grn_ctx *ctx, void *ptr, const char* file, int line);
+GRN_API void grn_free(grn_ctx *ctx, void *ptr, const char* file, int line);
 
 void grn_assert(grn_ctx *ctx, int cond, const char* file, int line, const char* func);
 
 /**** grn_ctx ****/
 
-extern grn_ctx grn_gctx;
+GRN_VAR grn_ctx grn_gctx;
 extern int grn_pagesize;
 extern grn_critical_section grn_glock;
 extern uint32_t grn_gtick;
 extern grn_obj *grn_true, *grn_false, *grn_null;
-extern const char *grn_log_path;
-extern const char *grn_qlog_path;
+GRN_VAR const char *grn_log_path;
+GRN_VAR const char *grn_qlog_path;
 
 #define GRN_CTX_ALLOCATED                            (0x80)
 #define GRN_CTX_TEMPORARY_DISABLE_II_RESOLVE_SEL_AND (0x40)
@@ -390,47 +390,84 @@ extern grn_timeval grn_starttime;
 #define GRN_TIMEVAL_STR_FORMAT "%04d-%02d-%02d %02d:%02d:%02d.%06d"
 #endif /* GRN_TIMEVAL_STR_FORMAT */
 
-grn_rc grn_timeval_now(grn_ctx *ctx, grn_timeval *tv);
-grn_rc grn_timeval2str(grn_ctx *ctx, grn_timeval *tv, char *buf);
+GRN_API grn_rc grn_timeval_now(grn_ctx *ctx, grn_timeval *tv);
+GRN_API grn_rc grn_timeval2str(grn_ctx *ctx, grn_timeval *tv, char *buf);
 grn_rc grn_str2timeval(const char *str, uint32_t str_len, grn_timeval *tv);
 
-typedef struct {
-  int32_t latitude;
-  int32_t longitude;
-} grn_geo_point;
-
-#define GRN_GEOPOINT_SET(ctx,obj,val) \
-  (grn_bulk_write_from((ctx), (obj), (char *)&val, 0, sizeof(grn_geo_point)))
-
-void grn_ctx_log(grn_ctx *ctx, char *fmt, ...);
+GRN_API void grn_ctx_log(grn_ctx *ctx, char *fmt, ...);
 void grn_ctx_qe_fin(grn_ctx *ctx);
 void grn_ctx_loader_clear(grn_ctx *ctx);
 void grn_log_reopen(grn_ctx *ctx);
 
-grn_rc grn_ctx_sendv(grn_ctx *ctx, int argc, char **argv, int flags);
+GRN_API grn_rc grn_ctx_sendv(grn_ctx *ctx, int argc, char **argv, int flags);
 void grn_ctx_set_next_expr(grn_ctx *ctx, grn_obj *expr);
 
 int grn_alloc_count(void);
 
 grn_content_type grn_get_ctype(grn_obj *var);
 
+/**** db_obj ****/
+
+/* flag values used for grn_obj.header.impl_flags */
+
+#define GRN_OBJ_ALLOCATED              (0x01<<2) /* allocated by ctx */
+#define GRN_OBJ_EXPRVALUE              (0x01<<3) /* value allocated by grn_expr */
+#define GRN_OBJ_EXPRCONST              (0x01<<4) /* constant allocated by grn_expr */
+
+typedef struct _grn_hook grn_hook;
+
+typedef struct {
+  grn_obj_header header;
+  grn_id range;  /* table: type of subrecords, column: type of values */
+  /* -- compatible with grn_accessor -- */
+  grn_id id;
+  grn_obj *db;
+  grn_user_data user_data;
+  grn_proc_func *finalizer;
+  grn_hook *hooks[5];
+  void *source;
+  uint32_t source_size;
+  uint32_t max_n_subrecs;
+  uint8_t subrec_size;
+  uint8_t subrec_offset;
+  uint8_t record_unit;
+  uint8_t subrec_unit;
+  //  grn_obj_flags flags;
+} grn_db_obj;
+
+#define GRN_DB_OBJ_SET_TYPE(db_obj,obj_type) {\
+  (db_obj)->obj.header.type = (obj_type);\
+  (db_obj)->obj.header.impl_flags = 0;\
+  (db_obj)->obj.header.flags = 0;\
+  (db_obj)->obj.id = GRN_ID_NIL;\
+  (db_obj)->obj.user_data.ptr = NULL;\
+  (db_obj)->obj.finalizer = NULL;\
+  (db_obj)->obj.hooks[0] = NULL;\
+  (db_obj)->obj.hooks[1] = NULL;\
+  (db_obj)->obj.hooks[2] = NULL;\
+  (db_obj)->obj.hooks[3] = NULL;\
+  (db_obj)->obj.hooks[4] = NULL;\
+  (db_obj)->obj.source = NULL;\
+  (db_obj)->obj.source_size = 0;\
+}
+
 /**** cache ****/
 
 void grn_cache_init(void);
 grn_obj *grn_cache_fetch(grn_ctx *ctx, const char *str, uint32_t str_size);
 void grn_cache_unref(const char *str, uint32_t str_size);
 void grn_cache_update(grn_ctx *ctx, const char *str, uint32_t str_size, grn_obj *value);
-void grn_cache_expire(uint32_t size);
+void grn_cache_expire(int32_t size);
 void grn_cache_fin(void);
 uint32_t *grn_cach_max_nentries(void);
 
 /**** receive handler ****/
 
-void grn_ctx_recv_handler_set(grn_ctx *c, void (*func)(grn_ctx *, int, void *),
-                              void *func_arg);
+GRN_API void grn_ctx_recv_handler_set(grn_ctx *c, void (*func)(grn_ctx *, int, void *),
+                                      void *func_arg);
 
 void grn_ctx_concat_func(grn_ctx *ctx, int flags, void *dummy);
-void grn_ctx_stream_out_func(grn_ctx *c, int flags, void *stream);
+GRN_API void grn_ctx_stream_out_func(grn_ctx *c, int flags, void *stream);
 
 grn_rc grn_db_init_builtin_procs(grn_ctx *ctx);
 
