@@ -22,9 +22,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifndef WIN32
+#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif /* WIN32 */
+#endif /* HAVE_CONFIG_H */
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -41,7 +41,7 @@
 #include "lib/com.h"
 
 #ifdef WIN32
-#include <Windows.h>
+#include <windows.h>
 #include <stddef.h>
 #else
 #include <sys/param.h>
@@ -61,12 +61,17 @@
 #define DEFAULT_PORT 10041
 #define DEFAULT_DEST "localhost"
 
+#define OUT_JSON 0
+#define OUT_TSV  1
+
+static int grntest_outtype = OUT_JSON;
 
 static grn_critical_section grntest_cs;
 
 static int grntest_stop_flag = 0;
 static int grntest_detail_on = 0;
 static int grntest_remote_mode = 0;
+static int grntest_localonly = 0;
 #define TMPFILE "_grntest.tmp"
 
 static grn_ctx grntest_server_context;
@@ -79,7 +84,7 @@ FILE *grntest_logfp;
 
 #ifdef WIN32
 typedef SOCKET ftpsocket;
-#define FTPERROR INVALID_SOCKET 
+#define FTPERROR INVALID_SOCKET
 #define ftpclose closesocket
 #define GROONGA_PATH "groonga.exe"
 PROCESS_INFORMATION grntest_pi;
@@ -315,7 +320,7 @@ escape_command(char *in, int ilen,  char *out, int olen)
 
 static
 int
-report_load_command(grn_ctx *ctx, char *ret, int task_id, long long int start_time, 
+report_load_command(grn_ctx *ctx, char *ret, int task_id, long long int start_time,
                     grn_obj *end_time)
 {
   int i, len;
@@ -342,15 +347,20 @@ report_load_command(grn_ctx *ctx, char *ret, int task_id, long long int start_ti
 
   start = start_time - GRN_TIME_VALUE(&grntest_starttime);
   end = GRN_TIME_VALUE(end_time) - GRN_TIME_VALUE(&grntest_starttime);
-  fprintf(grntest_logfp, "[%d, \"load\", %lld, %lld, %s],\n",  
-          task_id,  start, end, rettmp);
+  if (grntest_outtype == OUT_TSV) {
+    fprintf(grntest_logfp, "report\t%d\tload\t%lld\t%lld\t%s\n",
+            task_id,  start, end, rettmp);
+  } else {
+    fprintf(grntest_logfp, "[%d, \"load\", %lld, %lld, %s],\n",
+            task_id,  start, end, rettmp);
+  }
   fflush(grntest_logfp);
   return 0;
 }
 
 static
 int
-report_command(grn_ctx *ctx, char *command, char *ret, int task_id, 
+report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
                grn_obj *start_time, grn_obj *end_time)
 {
   int i, len, clen;
@@ -380,8 +390,13 @@ report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
   end = GRN_TIME_VALUE(end_time) - GRN_TIME_VALUE(&grntest_starttime);
   clen = strlen(command);
   escape_command(command, clen, command_escaped, clen * 2);
-  fprintf(grntest_logfp, "[%d, \"%s\", %lld, %lld, %s],\n",  
-          task_id, command_escaped, start, end, rettmp);
+  if (grntest_outtype == OUT_TSV) {
+    fprintf(grntest_logfp, "report\t%d\t%s\t%lld\t%lld\t%s\n",
+            task_id, command_escaped, start, end, rettmp);
+  } else {
+    fprintf(grntest_logfp, "[%d, \"%s\", %lld, %lld, %s],\n",
+            task_id, command_escaped, start, end, rettmp);
+  }
   fflush(grntest_logfp);
   return 0;
 }
@@ -401,8 +416,12 @@ output_result_final(grn_ctx *ctx, int qnum)
   self = latency;
   sec = self / (double)1000000;
   qps = (double)qnum / sec;
-  fprintf(grntest_logfp, 
-         "{\"total\": %lld, \"qps\": %f, \"queries\": %d}]\n", latency, qps, qnum);
+  if (grntest_outtype == OUT_TSV) {
+    fprintf(grntest_logfp, "total\t%lld\t%f\t%d\n", latency, qps, qnum);
+  } else {
+    fprintf(grntest_logfp,
+           "{\"total\": %lld, \"qps\": %f, \"queries\": %d}]\n", latency, qps, qnum);
+  }
   grn_obj_close(ctx, &end_time);
   return 0;
 }
@@ -411,10 +430,13 @@ static
 int
 output_sysinfo(char *sysinfo)
 {
-  fprintf(grntest_logfp, "[%s\n", sysinfo);
+  if (grntest_outtype == OUT_TSV) {
+    fprintf(grntest_logfp, "%s", sysinfo);
+  } else {
+    fprintf(grntest_logfp, "[%s\n", sysinfo);
+  }
   return 0;
 }
-  
 
 static
 int
@@ -440,7 +462,7 @@ static
 int
 error_command(grn_ctx *ctx, char *command, int task_id)
 {
-  fprintf(stderr, "error!:command=%s task_id = %d\n", command, task_id);
+  fprintf(stderr, "error!:command=[%s] task_id = %d\n", command, task_id);
   fflush(stderr);
   error_exit_in_thread(1);
   return 0;
@@ -462,11 +484,14 @@ diff_result(char *expect, int elen, char *result, int rlen)
     i++;
   }
   if (e == NULL) {
+/*
     fputc('[', stderr);
     fwrite(expect, 1, elen, stderr);
     fputc(']', stderr);
-    fprintf(stderr, " is not groonga command output");
+    fprintf(stderr, " is not groonga command output\n", );
     return 1;
+*/
+    e = expect;
   }
 
   i = 0;
@@ -478,19 +503,22 @@ diff_result(char *expect, int elen, char *result, int rlen)
     i++;
   }
   if (r == NULL) {
+/*
     fputc('[', stderr);
     fwrite(result, 1, rlen, stderr);
     fputc(']', stderr);
-    fprintf(stderr, " is not groonga command output");
+    fprintf(stderr, " is not groonga command output\n");
     return 1;
+*/
+    r = result;
   }
-  
+
   return strncmp(e, r, strlen(e));
 }
 
 static
 int
-do_load_command(grn_ctx *ctx, char *command, int type, int task_id, 
+do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
                 long long int *load_start)
 {
   char *res;
@@ -505,7 +533,7 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
   }
 
   grn_ctx_send(ctx, command, strlen(command), 0);
-/* fix me. 
+/* fix me.
    when command fails, ctx->rc is not 0 in local mode!
   if (ctx->rc) {
     fprintf(stderr, "ctx_send:rc=%d:command:%s\n", ctx->rc, command);
@@ -527,7 +555,7 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
       self = GRN_TIME_VALUE(&end_time) - *load_start;
 
       if (grntest_task[task_id].max < self) {
-        grntest_task[task_id].max = self; 
+        grntest_task[task_id].max = self;
       }
       if (grntest_task[task_id].min > self) {
         grntest_task[task_id].min = self;
@@ -555,7 +583,7 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
         int loglen;
         FILE *outfp = grntest_job[grntest_task[task_id].job_id].outputlog;
         logbuf[LOGBUF_LEN-2] = '\0';
-        if (fgets(logbuf, LOGBUF_LEN, 
+        if (fgets(logbuf, LOGBUF_LEN,
                   grntest_job[grntest_task[task_id].job_id].inputlog) == NULL) {
           fprintf(stderr, "Cannot get input-log\n");
           error_exit_in_thread(55);
@@ -602,7 +630,7 @@ do_command(grn_ctx *ctx, char *command, int type, int task_id)
   GRN_TIME_NOW(ctx, &start_time);
 
   grn_ctx_send(ctx, command, strlen(command), 0);
-/* fix me. 
+/* fix me.
    when command fails, ctx->rc is not 0 in local mode!
 */
   if (ctx->rc) {
@@ -625,7 +653,7 @@ do_command(grn_ctx *ctx, char *command, int type, int task_id)
       self = GRN_TIME_VALUE(&end_time) - GRN_TIME_VALUE(&start_time);
 
       if (grntest_task[task_id].max < self) {
-        grntest_task[task_id].max = self; 
+        grntest_task[task_id].max = self;
       }
       if (grntest_task[task_id].min > self) {
         grntest_task[task_id].min = self;
@@ -653,7 +681,7 @@ do_command(grn_ctx *ctx, char *command, int type, int task_id)
         int loglen;
         FILE *outfp = grntest_job[grntest_task[task_id].job_id].outputlog;
         logbuf[LOGBUF_LEN-2] = '\0';
-        if (fgets(logbuf, LOGBUF_LEN, 
+        if (fgets(logbuf, LOGBUF_LEN,
                   grntest_job[grntest_task[task_id].job_id].inputlog) == NULL) {
           fprintf(stderr, "Cannot get inputlog\n");
           error_exit_in_thread(55);
@@ -737,7 +765,7 @@ worker_sub(intptr_t task_id)
       if (!fp) {
         fprintf(stderr, "Cannot open %s\n",grntest_task[task_id].file);
         error_exit_in_thread(1);
-      } 
+      }
       tmpbuf[MAX_COMMAND_LEN-2] = '\0';
       load_mode = 0;
       load_count = 0;
@@ -751,12 +779,15 @@ worker_sub(intptr_t task_id)
         if (comment_p(tmpbuf)) {
           continue;
         }
+        if (tmpbuf[0] == '\0') {
+          continue;
+        }
         if (load_command_p(tmpbuf)) {
           load_mode = 1;
           load_count = 1;
         }
         if (load_mode == 1) {
-          if (do_load_command(&grntest_ctx[task_id], tmpbuf, 
+          if (do_load_command(&grntest_ctx[task_id], tmpbuf,
                               grntest_task[task_id].jobtype,
                               task_id, &load_start)) {
             grntest_task[task_id].qnum += load_count;
@@ -767,7 +798,7 @@ worker_sub(intptr_t task_id)
           load_count++;
           continue;
         }
-        do_command(&grntest_ctx[task_id], tmpbuf, 
+        do_command(&grntest_ctx[task_id], tmpbuf,
                      grntest_task[task_id].jobtype,
                      task_id);
         grntest_task[task_id].qnum++;
@@ -784,8 +815,8 @@ worker_sub(intptr_t task_id)
           load_mode = 1;
         }
         if (load_mode == 1) {
-          if (do_load_command(&grntest_ctx[task_id], 
-                              grntest_task[task_id].table->command[line], 
+          if (do_load_command(&grntest_ctx[task_id],
+                              grntest_task[task_id].table->command[line],
                               grntest_task[task_id].jobtype, task_id, &load_start)) {
             load_mode = 0;
             load_start = 0LL;
@@ -793,11 +824,11 @@ worker_sub(intptr_t task_id)
           }
           continue;
         }
-        do_command(&grntest_ctx[task_id], 
-                   grntest_task[task_id].table->command[line], 
+        do_command(&grntest_ctx[task_id],
+                   grntest_task[task_id].table->command[line],
                    grntest_task[task_id].jobtype, task_id);
         grntest_task[task_id].qnum++;
-      } 
+      }
     }
   }
 
@@ -816,33 +847,46 @@ worker_sub(intptr_t task_id)
 
   grntest_job[grntest_task[task_id].job_id].qnum += grntest_task[task_id].qnum;
   grntest_job[grntest_task[task_id].job_id].done++;
-  if (grntest_job[grntest_task[task_id].job_id].done == 
+  if (grntest_job[grntest_task[task_id].job_id].done ==
       grntest_job[grntest_task[task_id].job_id].concurrency) {
     char tmpbuf[BUF_LEN];
     sec = self / (double)1000000;
     qps = (double)grntest_job[grntest_task[task_id].job_id].qnum/ sec;
     grntest_jobdone++;
-    sprintf(tmpbuf, 
-            "{\"job\": \"%s\", \"latency\": %lld, \"self\": %lld, \"qps\": %f, \"min\": %lld, \"max\": %lld, \"queries\": %d}",
-            grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
-            grntest_job[grntest_task[task_id].job_id].min,
-            grntest_job[grntest_task[task_id].job_id].max,
-            grntest_job[grntest_task[task_id].job_id].qnum);
-    if (grntest_jobdone < grntest_jobnum) {
-      strcat(tmpbuf, ",");
+    if (grntest_outtype == OUT_TSV) {
+      sprintf(tmpbuf,
+              "job\t%s\t%lld\t%lld\t%f\t%lld\t%lld\t%d\n",
+              grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
+              grntest_job[grntest_task[task_id].job_id].min,
+              grntest_job[grntest_task[task_id].job_id].max,
+              grntest_job[grntest_task[task_id].job_id].qnum);
+    } else {
+      sprintf(tmpbuf,
+              "{\"job\": \"%s\", \"latency\": %lld, \"self\": %lld, \"qps\": %f, \"min\": %lld, \"max\": %lld, \"queries\": %d}",
+              grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
+              grntest_job[grntest_task[task_id].job_id].min,
+              grntest_job[grntest_task[task_id].job_id].max,
+              grntest_job[grntest_task[task_id].job_id].qnum);
+      if (grntest_jobdone < grntest_jobnum) {
+        strcat(tmpbuf, ",");
+      }
     }
     strcat(grntest_log_tmpbuf, tmpbuf);
     if (grntest_log_tmpbuf[LOGBUF_LEN - 2] != '\0') {
       error_exit_in_thread(1);
     }
     if (grntest_jobdone == grntest_jobnum) {
-      if (grntest_detail_on) {
-        fseek(grntest_logfp, -2, SEEK_CUR);
-        fprintf(grntest_logfp, "], \n");
+      if (grntest_outtype == OUT_TSV) {
+        fprintf(grntest_logfp, "%s", grntest_log_tmpbuf);
+      } else {
+        if (grntest_detail_on) {
+          fseek(grntest_logfp, -2, SEEK_CUR);
+          fprintf(grntest_logfp, "], \n");
+        }
+        fprintf(grntest_logfp, "\"summary\" :[");
+        fprintf(grntest_logfp, "%s", grntest_log_tmpbuf);
+        fprintf(grntest_logfp, "]");
       }
-      fprintf(grntest_logfp, "\"summary\" :[");
-      fprintf(grntest_logfp, "%s", grntest_log_tmpbuf);
-      fprintf(grntest_logfp, "]");
       fflush(grntest_logfp);
     }
   }
@@ -855,7 +899,7 @@ worker_sub(intptr_t task_id)
 #ifdef WIN32
 static
 int
-__stdcall 
+__stdcall
 worker(void *val)
 {
   worker_sub((intptr_t) val);
@@ -914,7 +958,7 @@ thread_main(grn_ctx *ctx, int num)
       error_exit_in_thread(1);
     }
   }
-  
+
   for (i = 0; i < num; i++) {
     ret = pthread_join(pthread[i], NULL);
     if (ret) {
@@ -951,64 +995,116 @@ get_sysinfo(const char *path, char *result, int olen)
   MEMORYSTATUSEX minfo;
   OSVERSIONINFO osinfo;
 
-  strcpy(result, "{");
-
-  sprintf(tmpbuf, "\"script\": \"%s.scr\",\n", grntest_scriptname);
-  strcat(result, tmpbuf);
-  sprintf(tmpbuf, "  \"user\": \"%s\",\n", grntest_username);
-  strcat(result, tmpbuf);
-  sprintf(tmpbuf, "  \"date\": \"%s\",\n", grntest_date);
-  strcat(result, tmpbuf);
+  if (grntest_outtype == OUT_TSV) {
+    result[0] = '\0';
+    sprintf(tmpbuf, "sctipt\t%s\n", grntest_scriptname);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "user\t%s\n", grntest_username);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "date\t%s\n", grntest_date);
+    strcat(result, tmpbuf);
+  } else {
+    strcpy(result, "{");
+    sprintf(tmpbuf, "\"script\": \"%s.scr\",\n", grntest_scriptname);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "  \"user\": \"%s\",\n", grntest_username);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "  \"date\": \"%s\",\n", grntest_date);
+    strcat(result, tmpbuf);
+  }
 
   memset(cpustring, 0, 64);
+#ifndef __GNUC__
   __cpuid(cinfo, 0x80000002);
   memcpy(cpustring, cinfo, 16);
   __cpuid(cinfo, 0x80000003);
   memcpy(cpustring+16, cinfo, 16);
   __cpuid(cinfo, 0x80000004);
   memcpy(cpustring+32, cinfo, 16);
+#endif
 
-  sprintf(tmpbuf, "  \"CPU\": \"%s\",\n", cpustring);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\n", cpustring);
+  } else {
+    sprintf(tmpbuf, "  \"CPU\": \"%s\",\n", cpustring);
+  }
   strcat(result, tmpbuf);
 
-  if (sizeof(int *) == 8 ) {
+  if (sizeof(int *) == 8) {
     grntest_osinfo = OS_WINDOWS64;
-    sprintf(tmpbuf, "  \"BIT\": 64,\n");
-    strcat(result, tmpbuf);
+    if (grntest_outtype == OUT_TSV) {
+      sprintf(tmpbuf, "64BIT\n");
+    } else {
+      sprintf(tmpbuf, "  \"BIT\": 64,\n");
+    }
   } else {
     grntest_osinfo = OS_WINDOWS32;
-    sprintf(tmpbuf, "  \"BIT\": 32,\n");
-    strcat(result, tmpbuf);
+    if (grntest_outtype == OUT_TSV) {
+      sprintf(tmpbuf, "32BIT\n");
+    } else {
+      sprintf(tmpbuf, "  \"BIT\": 32,\n");
+    }
   }
+  strcat(result, tmpbuf);
 
   GetSystemInfo(&sinfo);
-  sprintf(tmpbuf, "  \"CORE\": %d,\n", sinfo.dwNumberOfProcessors);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "CORE\t%d\n", sinfo.dwNumberOfProcessors);
+  } else {
+    sprintf(tmpbuf, "  \"CORE\": %d,\n", sinfo.dwNumberOfProcessors);
+  }
   strcat(result, tmpbuf);
 
   minfo.dwLength = sizeof(minfo);
   GlobalMemoryStatusEx(&minfo);
-  sprintf(tmpbuf, "  \"RAM\": \"%I64dMByte\",\n", 
-          minfo.ullTotalPhys/(1024*1024));
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "RAM\t%I64dMByte\n", minfo.ullTotalPhys/(1024*1024));
+  } else {
+    sprintf(tmpbuf, "  \"RAM\": \"%I64dMByte\",\n", minfo.ullTotalPhys/(1024*1024));
+  }
   strcat(result, tmpbuf);
 
   GetDiskFreeSpaceEx(NULL, NULL, &dinfo, NULL);
-  sprintf(tmpbuf, "  \"HDD\": \"%I64dKBytes\",\n", dinfo.QuadPart/1024 );
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "HDD\t%I64dKBytes\n", dinfo.QuadPart/1024 );
+  } else {
+    sprintf(tmpbuf, "  \"HDD\": \"%I64dKBytes\",\n", dinfo.QuadPart/1024 );
+  }
   strcat(result, tmpbuf);
 
   osinfo.dwOSVersionInfoSize = sizeof(osinfo); GetVersionEx(&osinfo);
-  sprintf(tmpbuf, "  \"OS\": \"Windows %d.%d\",\n", osinfo.dwMajorVersion,
-  osinfo.dwMinorVersion);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "Windows %d.%d\n", osinfo.dwMajorVersion, osinfo.dwMinorVersion);
+  } else {
+    sprintf(tmpbuf, "  \"OS\": \"Windows %d.%d\",\n", osinfo.dwMajorVersion,
+            osinfo.dwMinorVersion);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"HOST\": \"%s\",\n", grntest_serverhost);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\n", grntest_serverhost);
+  } else {
+    sprintf(tmpbuf, "  \"HOST\": \"%s\",\n", grntest_serverhost);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"PORT\": \"%d\",\n", grntest_serverport);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%d\n", grntest_serverport);
+  } else {
+    sprintf(tmpbuf, "  \"PORT\": \"%d\",\n", grntest_serverport);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"VERSION\": \"%s\"\n", grn_get_version());
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\"\n", grn_get_version());
+  } else {
+    sprintf(tmpbuf, "  \"VERSION\": \"%s\"\n", grn_get_version());
+  }
+
   strcat(result, tmpbuf);
-  strcat(result, "}");
+  if (grntest_outtype != OUT_TSV) {
+    strcat(result, "}");
+  }
 
 #else /* linux only */
   FILE *fp;
@@ -1019,14 +1115,23 @@ get_sysinfo(const char *path, char *result, int olen)
   struct utsname ubuf;
   struct statvfs vfsbuf;
 
-  strcpy(result, "{");
-
-  sprintf(tmpbuf, "\"script\": \"%s.scr\",\n", grntest_scriptname);
-  strcat(result, tmpbuf);
-  sprintf(tmpbuf, "  \"user\": \"%s\",\n", grntest_username);
-  strcat(result, tmpbuf);
-  sprintf(tmpbuf, "  \"date\": \"%s\",\n", grntest_date);
-  strcat(result, tmpbuf);
+  if (grntest_outtype == OUT_TSV) {
+    result[0] = '\0';
+    sprintf(tmpbuf, "sctipt\t%s\n", grntest_scriptname);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "user\t%s\n", grntest_username);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "date\t%s\n", grntest_date);
+    strcat(result, tmpbuf);
+  } else {
+    strcpy(result, "{");
+    sprintf(tmpbuf, "\"script\": \"%s.scr\",\n", grntest_scriptname);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "  \"user\": \"%s\",\n", grntest_username);
+    strcat(result, tmpbuf);
+    sprintf(tmpbuf, "  \"date\": \"%s\",\n", grntest_date);
+    strcat(result, tmpbuf);
+  }
 
   fp = fopen("/proc/cpuinfo", "r");
   if (!fp) {
@@ -1038,25 +1143,39 @@ get_sysinfo(const char *path, char *result, int olen)
     if (!strncmp(tmpbuf, "model name\t: ", 13)) {
       strcpy(cpustring, &tmpbuf[13]);
     }
-    if (!strncmp(tmpbuf, "processor\t: ", 12)) {
-      cpunum = grntest_atoi(&tmpbuf[12], &tmpbuf[12] + 20, NULL);
-    }
   }
   fclose(fp);
-  sprintf(tmpbuf, "  \"CPU\": %s\",\n", cpustring);
+  cpunum = sysconf(_SC_NPROCESSORS_CONF);
+
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\n", cpustring);
+  } else {
+    sprintf(tmpbuf, "  \"CPU\": %s\",\n", cpustring);
+  }
   strcat(result, tmpbuf);
 
-  if (sizeof(int *) == 8 ) {
+  if (sizeof(int *) == 8) {
     grntest_osinfo = OS_LINUX64;
-    sprintf(tmpbuf, "  \"BIT\": 64,\n");
-    strcat(result, tmpbuf);
+    if (grntest_outtype == OUT_TSV) {
+      sprintf(tmpbuf, "64BIT\n");
+    } else {
+      sprintf(tmpbuf, "  \"BIT\": 64,\n");
+    }
   } else {
     grntest_osinfo = OS_LINUX32;
-    sprintf(tmpbuf, "  \"BIT\": 32,\n");
-    strcat(result, tmpbuf);
+    if (grntest_outtype == OUT_TSV) {
+      sprintf(tmpbuf, "32BIT\n");
+    } else {
+      sprintf(tmpbuf, "  \"BIT\": 32,\n");
+    }
   }
+  strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"CORE\": %d,\n", cpunum + 1);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "CORE\t%d\n", cpunum);
+  } else {
+    sprintf(tmpbuf, "  \"CORE\": %d,\n", cpunum);
+  }
   strcat(result, tmpbuf);
 
   fp = fopen("/proc/meminfo", "r");
@@ -1071,7 +1190,11 @@ get_sysinfo(const char *path, char *result, int olen)
     }
   }
   fclose(fp);
-  sprintf(tmpbuf, "  \"RAM\": \"%dMBytes\",\n", minfo/1024);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%dMBytes\n", minfo/1024);
+  } else {
+    sprintf(tmpbuf, "  \"RAM\": \"%dMBytes\",\n", minfo/1024);
+  }
   strcat(result, tmpbuf);
 
   ret = statvfs(path, &vfsbuf);
@@ -1080,29 +1203,50 @@ get_sysinfo(const char *path, char *result, int olen)
     exit(1);
   }
 
-  sprintf(tmpbuf, "  \"HDD\": \"%ldKBytes\",\n", vfsbuf.f_blocks * 4);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%ldKBytes\n", vfsbuf.f_blocks * 4);
+  } else {
+    sprintf(tmpbuf, "  \"HDD\": \"%ldKBytes\",\n", vfsbuf.f_blocks * 4);
+  }
   strcat(result, tmpbuf);
 
   uname(&ubuf);
-  sprintf(tmpbuf, "  \"OS\": \"%s %s\",\n", ubuf.sysname, ubuf.release);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s %s\n", ubuf.sysname, ubuf.release);
+  } else {
+    sprintf(tmpbuf, "  \"OS\": \"%s %s\",\n", ubuf.sysname, ubuf.release);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"HOST\": \"%s\",\n", grntest_serverhost);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\n", grntest_serverhost);
+  } else {
+    sprintf(tmpbuf, "  \"HOST\": \"%s\",\n", grntest_serverhost);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"PORT\": \"%d\",\n", grntest_serverport);
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%d\n", grntest_serverport);
+  } else {
+    sprintf(tmpbuf, "  \"PORT\": \"%d\",\n", grntest_serverport);
+  }
   strcat(result, tmpbuf);
 
-  sprintf(tmpbuf, "  \"VERSION\": \"%s\"\n", grn_get_version());
+  if (grntest_outtype == OUT_TSV) {
+    sprintf(tmpbuf, "%s\n", grn_get_version());
+  } else {
+    sprintf(tmpbuf, "  \"VERSION\": \"%s\"\n", grn_get_version());
+  }
   strcat(result, tmpbuf);
 
-  strcat(result, "},");
+  if (grntest_outtype != OUT_TSV) {
+    strcat(result, "},");
+  }
 #endif /* WIN32 */
   if (strlen(result) >= olen) {
     fprintf(stderr, "buffer overrun in get_sysinfo!\n");
     exit(1);
   }
-
   return 0;
 }
 
@@ -1119,7 +1263,7 @@ start_server(const char *dbpath, int r)
 
   if (strlen(dbpath) > BUF_LEN - 100) {
     fprintf(stderr, "too long dbpath!\n");
-    exit(1); 
+    exit(1);
   }
 
   strcpy(tmpbuf, GROONGA_PATH);
@@ -1157,9 +1301,9 @@ start_server(const char *dbpath, int r)
     grntest_server_id = pid;
   }
 
-#endif /* WIN32 */ 
+#endif /* WIN32 */
 
-  return 0; 
+  return 0;
 }
 
 static
@@ -1290,7 +1434,7 @@ parse_line(char *buf, int start, int end, int num)
   }
   tmpbuf[j] ='\0';
   if (out_or_test) {
-    if (out_p(grntest_job[num].jobtype)) { 
+    if (out_p(grntest_job[num].jobtype)) {
       grntest_job[num].outputlog = fopen(tmpbuf, "wb");
       if (grntest_job[num].outputlog == NULL) {
         fprintf(stderr, "Cannot open %s\n", tmpbuf);
@@ -1445,7 +1589,7 @@ make_task_table(grn_ctx *ctx, int jobnum)
         while (fgets(tmpbuf, MAX_COMMAND_LEN, fp) != NULL) {
           if (tmpbuf[MAX_COMMAND_LEN-2] != '\0') {
             tmpbuf[MAX_COMMAND_LEN-2] = '\0';
-            fprintf(stderr, "Too long command in %s\n", 
+            fprintf(stderr, "Too long command in %s\n",
                    grntest_job[i].commandfile);
             fprintf(stderr, "line =%d:%s\n", line + 1, tmpbuf);
             error_exit(ctx, 1);
@@ -1456,6 +1600,9 @@ make_task_table(grn_ctx *ctx, int jobnum)
           if (comment_p(tmpbuf)) {
             continue;
           }
+          if (tmpbuf[0] == '\0') {
+            continue;
+          }
           ctable->command[line] = GRN_STRDUP(tmpbuf);
           if (ctable->command[line] == NULL) {
             fprintf(stderr, "Cannot alloc commandfile:%s\n",
@@ -1465,7 +1612,7 @@ make_task_table(grn_ctx *ctx, int jobnum)
           ctable->num = line;
           line++;
           if (line >= MAX_COMMAND) {
-            fprintf(stderr, "Too many commands in %s\n", 
+            fprintf(stderr, "Too many commands in %s\n",
                    grntest_job[i].commandfile);
             error_exit(ctx, 1);
           }
@@ -1523,7 +1670,7 @@ printf("%d:type =%d:file=%s:con=%d:ntimes=%d\n", i, grntest_job[i].jobtype,
     fprintf(stderr, "Logical error\n");
     error_exit(ctx, 9);
   }
-  
+
   grntest_detail_on = 0;
   for (i = 0; i < task_num; i++) {
     grn_ctx_init(&grntest_ctx[i], 0);
@@ -1542,7 +1689,12 @@ printf("%d:type =%d:file=%s:con=%d:ntimes=%d\n", i, grntest_job[i].jobtype,
     }
   }
   if (grntest_detail_on) {
-    fprintf(grntest_logfp, "\"detail\": [\n");
+    if (grntest_outtype == OUT_TSV) {
+      ;
+    }
+    else {
+      fprintf(grntest_logfp, "\"detail\": [\n");
+    }
     fflush(grntest_logfp);
   }
 
@@ -1557,13 +1709,13 @@ printf("%d:type =%d:file=%s:con=%d:ntimes=%d\n", i, grntest_job[i].jobtype,
 
   i = 0;
   while (i < task_num) {
-    int job_id; 
+    int job_id;
     if (grntest_task[i].table != NULL) {
       job_id = grntest_task[i].job_id;
       for (j = 0; j < grntest_task[i].table->num; j++) {
         GRN_FREE(grntest_task[i].table->command[j]);
       }
-      GRN_FREE(grntest_task[i].table); 
+      GRN_FREE(grntest_task[i].table);
       while (job_id == grntest_task[i].job_id) {
         i++;
       }
@@ -1622,9 +1774,17 @@ do_script(grn_ctx *ctx, const char *sfile)
     if (job_num > 0) {
       GRN_TIME_INIT(&grntest_jobs_start, 0);
       GRN_TIME_NOW(ctx, &grntest_jobs_start);
-      fprintf(grntest_logfp, "{\"jobs\": \"%s\",\n", buf);
+      if (grntest_outtype == OUT_TSV) {
+        fprintf(grntest_logfp, "jobs-start\t%s\n", buf);
+      } else {
+        fprintf(grntest_logfp, "{\"jobs\": \"%s\",\n", buf);
+      }
       qnum = do_jobs(ctx, job_num, line);
-      fprintf(grntest_logfp, "},\n");
+      if (grntest_outtype == OUT_TSV) {
+        fprintf(grntest_logfp, "jobs-end\t%s\n", buf);
+      } else {
+        fprintf(grntest_logfp, "},\n");
+      }
       qnum_total = qnum_total + qnum;
 
       grn_obj_close(ctx, &grntest_jobs_start);
@@ -1761,7 +1921,7 @@ ftp_list(ftpsocket data_socket)
     buf[ret] = '\0';
     fprintf(stdout, "%s", buf);
   }
-  
+
   return 0;
 }
 
@@ -1846,7 +2006,7 @@ open_socket(char *host, int port)
   server.sin_family = AF_INET;
   server.sin_port = htons(port);
   server.sin_addr = *(struct in_addr*)&inaddr;
-   
+
   sock = socket(AF_INET, SOCK_STREAM, 0);
   if (sock == -1) {
     fprintf(stderr, "socket error\n");
@@ -1871,7 +2031,7 @@ get_ftp_date(char *buf)
     }
   }
   buf++;
-  
+
   return buf;
 }
 
@@ -1889,12 +2049,12 @@ get_size(char *buf)
   }
   buf++;
   size = grntest_atoi(buf, buf + strlen(buf), NULL);
-  
+
   return size;
 }
 
 int
-ftp_sub(char *user, char *passwd, char *host, const char *filename, 
+ftp_sub(char *user, char *passwd, char *host, const char *filename,
          int mode, char *cd_dirname, char *retval)
 {
   int size = 0;
@@ -1902,7 +2062,7 @@ ftp_sub(char *user, char *passwd, char *host, const char *filename,
   ftpsocket command_socket, data_socket;
   int data_port;
   char data_host[BUF_LEN];
-  char send_mesg[BUF_LEN]; 
+  char send_mesg[BUF_LEN];
   char buf[BUF_LEN];
 #ifdef WIN32
   char base[BUF_LEN];
@@ -2087,7 +2247,7 @@ ftp_main(int argc, char **argv)
 {
   char val[BUF_LEN];
   val[0] = '\0';
-  ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, argv[2], 
+  ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, argv[2],
           grntest_atoi(argv[3], argv[3] + strlen(argv[3]), NULL), argv[4], val);
   if (val[0] != '\0') {
     printf("val=%s\n", val);
@@ -2121,18 +2281,26 @@ static
 int
 get_date(char *date, time_t *sec)
 {
-#ifdef WIN32
+#ifdef __GNUC__
+#  ifdef HAVE_LOCALTIME_R
+  struct tm result;
+  struct tm *tm = &result;
+  localtime_r(sec, tm);
+#  else /* HAVE_LOCALTIME_R */
+  struct tm *tm = localtime(sec);
+#  endif /* HAVE_LOCALTIME_R */
+#else /* __GNUC__ */
   struct tm tmbuf;
   struct tm *tm = &tmbuf;
   localtime_s(tm, sec);
-#else
-  struct tm *tm = localtime(sec);
-#endif /* WIN32 */
+#endif /* __GNUC__ */
+
 #ifdef WIN32
   strftime(date, 128, "%Y-%m-%d %H:%M:%S", tm);
 #else
   strftime(date, 128, "%F %T", tm);
 #endif /* WIN32 */
+
   return 1;
 }
 
@@ -2169,7 +2337,7 @@ get_tm_from_serverdate(char *serverdate, struct tm *tm)
   int res;
   int year, month, day, hour, minute, second;
 
-  res = sscanf(serverdate, "%4d%2d%2d%2d%2d%2d", 
+  res = sscanf(serverdate, "%4d%2d%2d%2d%2d%2d",
                &year, &month, &day, &hour, &minute, &second);
 
 /*
@@ -2204,7 +2372,7 @@ sync_sub(grn_ctx *ctx, const char *filename)
   time_t st, lt;
   struct tm stm;
 
-  ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_TIME, "data", 
+  ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_TIME, "data",
                serverdate);
   if (ret == 0) {
     fprintf(stderr, "[%s] does not exist in server\n", filename);
@@ -2231,14 +2399,14 @@ sync_sub(grn_ctx *ctx, const char *filename)
     if (lt < st) {
       fprintf(stderr, "newer [%s] exists in server\n", filename);
       fflush(stderr);
-      ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_GET, "data", 
+      ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_GET, "data",
                     NULL);
       return ret;
     }
   } else {
     fprintf(stderr, "[%s] does not exist in local\n", filename);
     fflush(stderr);
-    ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_GET, "data", 
+    ret = ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, filename, MODE_GET, "data",
                   NULL);
     return ret;
   }
@@ -2305,7 +2473,7 @@ printf("commandfile=[%s]:buf=%s\n", grntest_job[i].commandfile, buf);
       fflush(stderr);
     }
     GRN_FREE(filelist[i]);
-  } 
+  }
   fclose(fp);
   return fnum;
 }
@@ -2331,14 +2499,16 @@ static
 void
 usage(void)
 {
-  fprintf(stderr, 
+  fprintf(stderr,
          "Usage: grntest [options...] [script] [db]\n"
          "options:\n"
-         "  -i, --host <ip/hostname>:  server address to listen (default: %s)\n"
-         "  -p, --port <port number>:  server port number (default: %d)\n"
          "  --dir:                     show script files on ftp server\n"
+         "  -i, --host <ip/hostname>:  server address to listen (default: %s)\n"
+         "  --localonly:               omit server connection\n"
+         "  --log-output-dir:          specify output dir (default: current)\n"
          "  --noftp:                   omit ftp connection\n"
-         "  --log-output-dir:          specify output dir (default: current)\n",
+         "  --output-type <tsv/json>:  specify output-type (default: json)\n"
+         "  -p, --port <port number>:  server port number (default: %d)\n",
          DEFAULT_DEST, DEFAULT_PORT);
   exit(1);
 }
@@ -2348,6 +2518,7 @@ enum {
   mode_list,
   mode_noftp,
   mode_usage,
+  mode_localonly,
 };
 
 static
@@ -2401,7 +2572,7 @@ get_token(char *line, char *token, int maxlen, char **next)
       token[i] = '\0';
       return 1;
     }
-    
+
     line++;
   }
   return 0;
@@ -2433,7 +2604,7 @@ check_script(const char *scrname)
     tmpbuf[strlen(tmpbuf)-1] = '\0';
     get_token(tmpbuf, token, BUF_LEN, &next);
     strcpy(prev, token);
-    
+
     while (next) {
       get_token(next, token, BUF_LEN, &next);
       if (!strncmp(prev, "SET_PORT", 8)) {
@@ -2472,7 +2643,7 @@ setsigalarm(int sec)
   sig.sa_handler = timeout;
   sig.sa_flags = 0;
   sigemptyset(&sig.sa_mask);
-  ret = sigaction(SIGALRM, &sig, NULL);   
+  ret = sigaction(SIGALRM, &sig, NULL);
   if (ret == -1) {
     fprintf(stderr, "setsigalarm:errno= %d\n", errno);
   }
@@ -2487,22 +2658,25 @@ main(int argc, char **argv)
   grn_ctx context;
   char sysinfo[BUF_LEN];
   char log[BUF_LEN];
-  const char *portstr=NULL, *hoststr=NULL, *dbname=NULL, *scrname=NULL, *outdir=NULL;
+  const char *portstr=NULL, *hoststr=NULL, *dbname=NULL, *scrname=NULL, *outdir=NULL, *outtype=NULL;
   time_t sec;
 
   static grn_str_getopt_opt opts[] = {
     {'i', "host", NULL, 0, getopt_op_none},
     {'p', "port", NULL, 0, getopt_op_none},
     {'\0', "log-output-dir", NULL, 0, getopt_op_none},
+    {'\0', "output-type", NULL, 0, getopt_op_none},
     {'\0', "dir", NULL, mode_list, getopt_op_update},
     {'\0', "noftp", NULL, mode_noftp, getopt_op_update},
     {'h', "help", NULL, mode_usage, getopt_op_update},
+    {'\0', "localonly", NULL, mode_localonly, getopt_op_update},
     {'\0', NULL, NULL, 0, 0}
   };
 
   opts[0].arg = &hoststr;
   opts[1].arg = &portstr;
   opts[2].arg = &outdir;
+  opts[3].arg = &outtype;
 
   i = grn_str_getopt(argc, argv, opts, &mode);
   if (i < 0) {
@@ -2520,15 +2694,19 @@ main(int argc, char **argv)
   }
 
   if (mode == mode_list) {
-    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, "*.scr", 1, "data", 
+    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, "*.scr", 1, "data",
              NULL);
     return 0;
   }
+  if (mode == mode_localonly) {
+    grntest_localonly = 1;
+    grntest_remote_mode = 1;
+  }
 
   if ((scrname == NULL) || (dbname == NULL)) {
     usage();
   }
-  
+
   strcpy(grntest_serverhost, DEFAULT_DEST);
   if (hoststr) {
     grntest_remote_mode = 1;
@@ -2539,9 +2717,13 @@ main(int argc, char **argv)
     grntest_serverport = grn_atoi(portstr, portstr + strlen(portstr), NULL);
   }
 
+  if (outtype && !strcmp(outtype, "tsv")) {
+    grntest_outtype = OUT_TSV;
+  }
+
   grn_init();
   CRITICAL_SECTION_INIT(grntest_cs);
-  
+
   grn_ctx_init(&context, 0);
   grn_ctx_init(&grntest_server_context, 0);
   grn_set_default_encoding(GRN_ENC_UTF8);
@@ -2556,8 +2738,10 @@ main(int argc, char **argv)
     start_server(dbname, 0);
   }
 
-  if (check_server(&grntest_server_context)) {
-    goto exit;
+  if (!grntest_localonly) {
+    if (check_server(&grntest_server_context)) {
+      goto exit;
+    }
   }
 
   get_scriptname(scrname, grntest_scriptname, ".scr");
@@ -2567,13 +2751,13 @@ main(int argc, char **argv)
   GRN_TIME_NOW(&context, &grntest_starttime);
   sec = (time_t)(GRN_TIME_VALUE(&grntest_starttime)/1000000);
   get_date(grntest_date, &sec);
-  
+
   if (outdir) {
-    sprintf(log, "%s/%s-%s-%lld.log", outdir, grntest_scriptname, 
-            grntest_username, GRN_TIME_VALUE(&grntest_starttime));
+    sprintf(log, "%s/%s-%s-%lld-%s.log", outdir, grntest_scriptname,
+            grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
   } else {
-    sprintf(log, "%s-%s-%lld.log", grntest_scriptname, 
-            grntest_username, GRN_TIME_VALUE(&grntest_starttime));
+    sprintf(log, "%s-%s-%lld-%s.log", grntest_scriptname,
+            grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
   }
 
   grntest_logfp = fopen(log, "w+b");
@@ -2590,7 +2774,7 @@ main(int argc, char **argv)
   fclose(grntest_logfp);
 
   if (mode != mode_noftp) {
-    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, log, 3, 
+    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, log, 3,
             "report", NULL);
   }
   fprintf(stderr, "grntest done. logfile=%s\n", log);
