@@ -20,6 +20,7 @@
 #include "lib/com.h"
 #include "lib/ql.h"
 #include "lib/proc.h"
+#include "lib/db.h"
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_SYS_WAIT_H
@@ -52,6 +53,7 @@ static int newdb;
 static int useql;
 static int (*do_client)(int argc, char **argv);
 static int (*do_server)(char *path);
+static uint32_t default_max_nfthreads = DEFAULT_MAX_NFTHREADS;
 
 static void
 usage(void)
@@ -81,7 +83,7 @@ usage(void)
           "  <db pathname> [<command>]: when standalone/server mode\n"
           "  <dest hostname>: when client mode (default: \"%s\")\n",
           hostname,
-          DEFAULT_PORT, DEFAULT_MAX_NFTHREADS, DEFAULT_DEST);
+          DEFAULT_PORT, default_max_nfthreads, DEFAULT_DEST);
 }
 
 static void
@@ -154,23 +156,25 @@ do_alone(int argc, char **argv)
   if (db) {
     grn_ctx_recv_handler_set(ctx, grn_ctx_stream_out_func, stdout);
     if (!argc) {
-      char *buf = GRN_MALLOC(BUFSIZE);
-      if (buf) {
+      grn_obj text;
+      GRN_TEXT_INIT(&text, 0);
+      rc = grn_bulk_reserve(ctx, &text, BUFSIZE);
+      if (!rc) {
+        char *buf = GRN_TEXT_VALUE(&text);
         while ((prompt(), fgets(buf, BUFSIZE, stdin))) {
           uint32_t size = strlen(buf) - 1;
           buf[size] = '\0';
           grn_ctx_send(ctx, buf, size, 0);
           if (ctx->stat == GRN_CTX_QUIT) { break; }
         }
-        GRN_FREE(buf);
-        rc = 0;
       } else {
-        fprintf(stderr, "grn_malloc failed (%d)\n", BUFSIZE);
+        fprintf(stderr, "grn_bulk_reserve() failed (%d): %d\n", BUFSIZE, rc);
       }
+      grn_obj_unlink(ctx, &text);
     } else {
       grn_ctx_sendv(ctx, argc, argv, 0);
     }
-    grn_db_close(ctx, db);
+    grn_obj_close(ctx, db);
   } else {
     fprintf(stderr, "db open failed (%s)\n", path);
   }
@@ -221,8 +225,11 @@ g_client(int argc, char **argv)
   grn_ctx_init(ctx, (batchmode ? GRN_CTX_BATCH_MODE : 0));
   if (!grn_ctx_connect(ctx, hostname, port, 0)) {
     if (!argc) {
-      char *buf = GRN_MALLOC(BUFSIZE);
-      if (buf) {
+      grn_obj text;
+      GRN_TEXT_INIT(&text, 0);
+      rc = grn_bulk_reserve(ctx, &text, BUFSIZE);
+      if (!rc) {
+        char *buf = GRN_TEXT_VALUE(&text);
         if (batchmode) { BATCHMODE(ctx); }
         while ((prompt(), fgets(buf, BUFSIZE, stdin))) {
           uint32_t size = strlen(buf) - 1;
@@ -231,11 +238,10 @@ g_client(int argc, char **argv)
           if (recvput(ctx)) { goto exit; }
           if (ctx->stat == GRN_CTX_QUIT) { break; }
         }
-        GRN_FREE(buf);
-        rc = 0;
       } else {
-        fprintf(stderr, "grn_malloc failed (%d)\n", BUFSIZE);
+        fprintf(stderr, "grn_bulk_reserve() failed (%d): %d\n", BUFSIZE, rc);
       }
+      grn_obj_unlink(ctx, &text);
     } else {
       grn_ctx_sendv(ctx, argc, argv, 0);
       if (recvput(ctx)) { goto exit; }
@@ -379,7 +385,7 @@ do_htreq(grn_ctx *ctx, grn_msg *msg, grn_obj *body)
     }
     /* TODO: handle post body */
     {
-      grn_rc expr_rc;
+      grn_rc expr_rc = GRN_SUCCESS;
       grn_obj jsonp_func;
       const char *mime_type;
 
@@ -1079,7 +1085,7 @@ static grn_com_queue ctx_new;
 static grn_com_queue ctx_old;
 static grn_mutex q_mutex;
 static grn_cond q_cond;
-static uint32_t nthreads = 0, nfthreads = 0, max_nfthreads = DEFAULT_MAX_NFTHREADS;
+static uint32_t nthreads = 0, nfthreads = 0, max_nfthreads;
 
 static void * CALLBACK
 h_worker(void *arg)
@@ -1234,7 +1240,7 @@ h_server(char *path)
         }
         grn_edges_fin(ctx);
       }
-      grn_db_close(ctx, db);
+      grn_obj_close(ctx, db);
     } else {
       fprintf(stderr, "db open failed (%s)\n", path);
     }
@@ -1472,7 +1478,7 @@ g_server(char *path)
         }
         grn_edges_fin(ctx);
       }
-      grn_db_close(ctx, db);
+      grn_obj_close(ctx, db);
     } else {
       fprintf(stderr, "db open failed (%s)\n", path);
     }
@@ -1535,6 +1541,18 @@ enum {
   grn_logger_info_set(&grn_gctx, &info);\
 } while(0)
 
+uint32_t
+get_core_number(void)
+{
+#ifdef WIN32
+  SYSTEM_INFO sinfo;
+  GetSystemInfo(&sinfo);
+  return sinfo.dwNumberOfProcessors;
+#else /* WIN32 */
+  return sysconf(_SC_NPROCESSORS_CONF);
+#endif /* WIN32 */
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1572,6 +1590,9 @@ main(int argc, char **argv)
   opts[13].arg = &protocol;
   opts[15].arg = &grn_log_path;
   opts[16].arg = &grn_qlog_path;
+  if (!(default_max_nfthreads = get_core_number())) {
+    default_max_nfthreads = DEFAULT_MAX_NFTHREADS;
+  }
   i = grn_str_getopt(argc, argv, opts, &mode);
   if (i < 0) { mode = mode_usage; }
   if (portstr) { port = atoi(portstr); }
@@ -1637,12 +1658,15 @@ main(int argc, char **argv)
   }
   if (max_nfthreadsstr) {
     max_nfthreads = atoi(max_nfthreadsstr);
+  } else {
+    max_nfthreads = default_max_nfthreads;
   }
   batchmode = !isatty(0);
   if (grn_init()) { return -1; }
   grn_set_default_encoding(enc);
   if (loglevel) { SET_LOGLEVEL(atoi(loglevel)); }
   grn_set_segv_handler();
+  grn_set_int_handler();
   if (hostnamestr) {
     size_t hostnamelen = strlen(hostnamestr);
     if (hostnamelen > HOST_NAME_MAX) {
